@@ -1,11 +1,11 @@
 /*
- * $Id: dbaccess.c,v 1.24 2002/05/27 14:43:34 alexis Exp $
+ * $Id: dbaccess.c,v 1.39.2.1 2003/07/21 13:51:34 erik Exp $
  *
  * dbaccess.c -- access methods for nsd(8) database
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -38,34 +38,28 @@
  *
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifdef	USE_MMAP
-#include <sys/mman.h>
-#endif
-
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 #include <unistd.h>
 #include <fcntl.h>
 
 #include "namedb.h"
+#include "util.h"
 
-#ifndef	USE_BERKELEY_DB
-
-int
-domaincmp(a, b)
-	register u_char *a;
-	register u_char *b;
+int 
+domaincmp (const void *left, const void *right)
 {
-	register int r;
-	register int alen = (int)*a;
-	register int blen = (int)*b;
+	int r;
+	const uint8_t *a = left;
+	const uint8_t *b = right;
+	int alen = (int)*a;
+	int blen = (int)*b;
 
 	while(alen && blen) {
 		a++; b++;
@@ -75,63 +69,16 @@ domaincmp(a, b)
 	return alen - blen;
 }
 
-#ifdef	USE_HEAP_HASH
-
-unsigned long
-domainhash(dname)
-	register u_char *dname;
-{
-        register unsigned long hash = 0;
-	register u_char *p = dname;
-
-	dname += *dname + 1;
-
-        while (p < dname)
-                hash = hash * 31 + *p++;
-        return hash;
-}
-
-#endif
-
-#endif
-
 struct domain *
-namedb_lookup(db, dname)
-	struct namedb *db;
-	u_char *dname;
+namedb_lookup (struct namedb *db, const uint8_t *dname)
 {
-#ifdef USE_BERKELEY_DB
-	DBT key, data;
-
-	bzero(&key, sizeof(key));
-	bzero(&data, sizeof(data));
-	key.size = (size_t)*dname;
-	key.data = dname + 1;
-
-	switch(db->db->get(db->db, NULL, &key, &data, 0)) {
-	case -1:
-		syslog(LOG_ERR, "database lookup failed: %m");
-		return NULL;
-	case DB_NOTFOUND:
-		return NULL;
-	case 0:
-		return data.data;
-	}
-
-	return NULL;
-
-#else 
 	return (struct domain *)heap_search(db->heap, dname);
-
-#endif /* USE_BERKELEY_DB */
 }
 
-struct answer *
-namedb_answer(d, type)
-	struct domain *d;
-	u_int16_t type;
+const struct answer *
+namedb_answer (const struct domain *d, int type)
 {
-	struct answer *a;
+	const struct answer *a;
 
 	DOMAIN_WALK(d, a) {
 		if(a->type == type) {
@@ -142,18 +89,13 @@ namedb_answer(d, type)
 }
 
 struct namedb *
-namedb_open(filename)
-	char *filename;
+namedb_open (const char *filename)
 {
 	struct namedb *db;
 	char magic[NAMEDB_MAGIC_SIZE] = NAMEDB_MAGIC;
 
-#ifdef	USE_BERKELEY_DB
-	DBT key, data;
-#else
 	char *p;
 	struct stat st;
-#endif
 
 	/* Allocate memory for it... */
 	if((db = xalloc(sizeof(struct namedb))) == NULL) {
@@ -166,44 +108,6 @@ namedb_open(filename)
 		return NULL;
 	}
 
-#ifdef USE_BERKELEY_DB
-	/* Setup the name database... */
-	if(db_create(&db->db, NULL, 0) != 0) {
-		free(db->filename);
-		free(db);
-		return NULL;
-	}
-
-	/* Open the database... */
-	if(db->db->open(db->db, db->filename, NULL, DB_UNKNOWN, DB_RDONLY, 0664) != 0) {
-		namedb_close(db);
-		return NULL;
-	}
-
-	/* Read the bitmasks... */
-	bzero(&key, sizeof(key));
-	bzero(&data, sizeof(data));
-
-	key.size = 0;
-	key.data = NULL;
-	if(db->db->get(db->db, NULL, &key, &data, 0) != 0) {
-		namedb_close(db);
-		return NULL;
-	}
-
-	if((data.size != (NAMEDB_BITMASKLEN * 3 + NAMEDB_MAGIC_SIZE)) ||
-		bcmp(data.data, magic, NAMEDB_MAGIC_SIZE)) {
-		syslog(LOG_ERR, "corrupted superblock in %s", db->filename);
-		namedb_close(db);
-		return NULL;
-	}
-
-	bcopy(data.data + NAMEDB_MAGIC_SIZE, db->masks[NAMEDB_AUTHMASK], NAMEDB_BITMASKLEN);
-	bcopy(data.data + NAMEDB_MAGIC_SIZE + NAMEDB_BITMASKLEN, db->masks[NAMEDB_STARMASK], NAMEDB_BITMASKLEN);
-	bcopy(data.data + NAMEDB_MAGIC_SIZE + NAMEDB_BITMASKLEN * 2, db->masks[NAMEDB_DATAMASK], NAMEDB_BITMASKLEN);
-
-#else
-
 	/* Open it... */
 	if((db->fd = open(db->filename, O_RDONLY)) == -1) {
 		free(db->filename);
@@ -221,15 +125,7 @@ namedb_open(filename)
 	/* What its size? */
 	db->mpoolsz = st.st_size;
 
-#ifdef	USE_MMAP
-	if((db->mpool = mmap(NULL, db->mpoolsz, PROT_READ, MAP_PRIVATE, db->fd, 0)) == MAP_FAILED) {
-		free(db->filename);
-		free(db);
-		return NULL;
-	}
-#else
-
-	if((db->mpool = malloc(db->mpoolsz)) == NULL) {
+	if((db->mpool = xalloc(db->mpoolsz)) == NULL) {
 		free(db->filename);
 		free(db);
 		return NULL;
@@ -241,17 +137,10 @@ namedb_open(filename)
 		free(db);
 		return NULL;
 	}
-#endif	/* USE_MMAP */
 
 	(void)close(db->fd);
 
-#ifdef USE_HEAP_RBTREE
-	if((db->heap = heap_create(malloc, domaincmp)) == NULL) {
-#else
-# ifdef USE_HEAP_HASH
-	if((db->heap = heap_create(malloc, domaincmp, domainhash, NAMEDB_HASH_SIZE)) == NULL) {
-# endif
-#endif
+	if((db->heap = heap_create(xalloc, domaincmp)) == NULL) {
 		free(db->mpool);
 		free(db->filename);
 		free(db);
@@ -260,23 +149,23 @@ namedb_open(filename)
 
 	p = db->mpool;
 
-	if(bcmp(p, magic, NAMEDB_MAGIC_SIZE)) {
-		syslog(LOG_ERR, "corrupted database: %s", db->filename);
+	if(memcmp(p, magic, NAMEDB_MAGIC_SIZE)) {
+		log_msg(LOG_ERR, "corrupted database: %s", db->filename);
 		namedb_close(db);
 		return NULL;
 	}
 	p += NAMEDB_MAGIC_SIZE;
 
 	while(*p) {
-		if(heap_insert(db->heap, p, p + ((*p + 1 +3) & 0xfffffffc), 1) == NULL) {
-			syslog(LOG_ERR, "failed to insert a domain: %m");
+		if(heap_insert(db->heap, p, p + ALIGN_UP(*p + 1), 1) == NULL) {
+			log_msg(LOG_ERR, "failed to insert a domain: %s", strerror(errno));
 			namedb_close(db);
 			return NULL;
 		}
-		p += (((u_int32_t)*p + 1 + 3) & 0xfffffffc);
-		p += *((u_int32_t *)p);
+		p += ALIGN_UP(*p + 1);
+		p += *((uint32_t *)p);
 		if(p > (db->mpool + db->mpoolsz)) {
-			syslog(LOG_ERR, "corrupted database %s", db->filename);
+			log_msg(LOG_ERR, "corrupted database %s", db->filename);
 			namedb_close(db);
 			errno = EINVAL;
 			return NULL;
@@ -285,47 +174,31 @@ namedb_open(filename)
 
 	p++;
 
-	if(bcmp(p, magic, NAMEDB_MAGIC_SIZE)) {
-		syslog(LOG_ERR, "corrupted database: %s", db->filename);
+	if(memcmp(p, magic, NAMEDB_MAGIC_SIZE)) {
+		log_msg(LOG_ERR, "corrupted database: %s", db->filename);
 		namedb_close(db);
 		return NULL;
 	}
 	p += NAMEDB_MAGIC_SIZE;
 
 	/* Copy the bitmasks... */
-	bcopy(p, db->masks[NAMEDB_AUTHMASK], NAMEDB_BITMASKLEN);
-	bcopy(p + NAMEDB_BITMASKLEN, db->masks[NAMEDB_STARMASK], NAMEDB_BITMASKLEN);
-	bcopy(p + NAMEDB_BITMASKLEN * 2, db->masks[NAMEDB_DATAMASK], NAMEDB_BITMASKLEN);
+	memcpy(db->masks[NAMEDB_AUTHMASK], p, NAMEDB_BITMASKLEN);
+	memcpy(db->masks[NAMEDB_STARMASK], p + NAMEDB_BITMASKLEN, NAMEDB_BITMASKLEN);
+	memcpy(db->masks[NAMEDB_DATAMASK], p + NAMEDB_BITMASKLEN * 2, NAMEDB_BITMASKLEN);
 
-#endif
+	log_msg(LOG_WARNING, "loaded %s, %lu entries", db->filename, db->heap->count);
 
-#if !defined(USE_BERKELEY_DB)
-#if defined(USE_HEAP_HASH)
-	syslog(LOG_WARNING, "loaded %s, %lu entries %lu hash collisions", db->filename,
-		db->heap->count, db->heap->collisions);
-#else 
-	syslog(LOG_WARNING, "loaded %s, %lu entries", db->filename, db->heap->count);
-#endif
-#else
-	syslog(LOG_WARNING, "loaded %s", db->filename);
-#endif
 	return db;
 }
 
 void
-namedb_close(db)
-	struct namedb *db;
+namedb_close (struct namedb *db)
 {
-#ifdef	USE_BERKELEY_DB
-	db->db->close(db->db, 0);
-#else
+	/* If it is already closed... */
+	if(db == NULL)
+		return;
 	heap_destroy(db->heap, 0, 0);
-#ifdef	USE_MMAP
-	munmap(db->mpool, db->mpoolsz);
-#else
 	free(db->mpool);
-#endif	/* USE_MMAP */
-#endif
 	if(db->filename)
 		free(db->filename);
 	free(db);
