@@ -1,11 +1,11 @@
 /*
- * $Id: namedb.h,v 1.27 2002/05/30 13:55:52 alexis Exp $
+ * $Id: namedb.h,v 1.38.2.1 2003/07/21 13:51:45 erik Exp $
  *
  * namedb.h -- nsd(8) internal namespace database definitions
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -41,11 +41,13 @@
 #ifndef _NAMEDB_H_
 #define	_NAMEDB_H_
 
+#define DEFAULT_ALIGNMENT       (sizeof (void *))
+#define ALIGN_UP(x)                (((x) + DEFAULT_ALIGNMENT - 1) & (~(DEFAULT_ALIGNMENT - 1)))
+
 #define	NAMEDB_MAXDSIZE		32768	/* Maximum size of a domain */
 
 #define	NAMEDB_DELEGATION	0x0001
 #define	NAMEDB_STEALTH		0x0002
-#define	NAMEDB_HASH_SIZE	65521
 
 #define	NAMEDB_AUTHMASK		0
 #define	NAMEDB_STARMASK		1
@@ -74,41 +76,44 @@
 #define	ANSWER_DATALEN(a)	a->datalen
 #define	ANSWER_DATALEN_PTR(a)	(&a->datalen)
 #define	ANSWER_END_PTR(a)	((struct answer *)a+1)
-#define	ANSWER_PTRS_PTR(a)	((u_int16_t *)ANSWER_END_PTR(a))
-#define	ANSWER_PTRS(a, i)	*((u_int16_t *)ANSWER_END_PTR(a) + (i))
-#define	ANSWER_RRS_PTR(a)	((u_int16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)
-#define	ANSWER_RRS(a, i)	(*(((u_int16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)+(i)) & ~NAMEDB_RRSET_COLOR)
-#define	ANSWER_RRS_COLOR(a, i)	(*(((u_int16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)+(i)) & NAMEDB_RRSET_COLOR)
-#define	ANSWER_DATA_PTR(a)	(u_char *)(((u_int16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)+ANSWER_RRSLEN(a))
+#define	ANSWER_PTRS_PTR(a)	((uint16_t *)ANSWER_END_PTR(a))
+#define	ANSWER_PTRS(a, i)	*((uint16_t *)ANSWER_END_PTR(a) + (i))
+#define	ANSWER_RRS_PTR(a)	((uint16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)
+#define	ANSWER_RRS(a, i)	(*(((uint16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)+(i)) & ~NAMEDB_RRSET_COLOR)
+#define	ANSWER_RRS_COLOR(a, i)	(*(((uint16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)+(i)) & NAMEDB_RRSET_COLOR)
+#define	ANSWER_DATA_PTR(a)	(uint8_t *)(((uint16_t *)ANSWER_END_PTR(a))+ANSWER_PTRSLEN(a)+ANSWER_RRSLEN(a))
 
 
 struct answer {
-	u_int32_t size;
-	u_int16_t type;
-	u_int16_t ancount;
-	u_int16_t nscount;
-	u_int16_t arcount;
-	u_int16_t ptrslen;
-	u_int16_t rrslen;
-	u_int32_t datalen;
-	/* u_int16_t ptrs[0]; */
-	/* u_int16_t rrs[0]; */
+	uint32_t size;
+	uint16_t type;
+	uint16_t ancount;
+	uint16_t nscount;
+	uint16_t arcount;
+	uint16_t ptrslen;
+	uint16_t rrslen;
+	uint32_t datalen;
+	/* uint16_t ptrs[0]; */
+	/* uint16_t rrs[0]; */
 	/* char *data; */
 };
 
 struct domain {
-	u_int32_t size;
-	u_int16_t	flags;
+	uint32_t size;
+	uint16_t flags;
+	void     *runtime_data; /* Additional run-time data (used for plugins) */
 };
 
-#define	NAMEDB_MAGIC		"NSDdbV01"
+#define	NAMEDB_MAGIC		"NSDdbV02"
 #define	NAMEDB_MAGIC_SIZE	8
 
-#define	NAMEDB_RRSET_WHITE	0x8000
-#define	NAMEDB_RRSET_BLACK	0x0000
-#define	NAMEDB_RRSET_COLOR	0x8000
+#define	NAMEDB_RRSET_WHITE	0x8000U
+#define	NAMEDB_RRSET_BLACK	0x0000U
+#define	NAMEDB_RRSET_COLOR	0x8000U
 
-#define	DOMAIN_WALK(d, a)	for(a = (struct answer *)(d + 1); ANSWER_SIZE(a) != 0; ((char *)a) += ANSWER_SIZE(a))
+#define	DOMAIN_WALK(d, a)	for(a = (struct answer *)(d + 1); \
+					ANSWER_SIZE(a) != 0; \
+					a = (struct answer *)((char *)a + ANSWER_SIZE(a)))
 #define	DOMAIN_SIZE(d)		d->size
 #define	DOMAIN_FLAGS(d)		d->flags
 
@@ -119,45 +124,30 @@ struct domain {
 #endif
 
 
-#ifdef	USE_BERKELEY_DB
-
-#include <db.h>
-
-struct namedb {
-	DB *db;
-	u_char masks[3][NAMEDB_BITMASKLEN];
-	char *filename;
-};
-
-#else
-
 #include "heap.h"
 
 struct namedb {
 	heap_t *heap;
-	u_char masks[3][NAMEDB_BITMASKLEN];
+	uint8_t masks[3][NAMEDB_BITMASKLEN];
 	char *mpool;
 	size_t	mpoolsz;
 	char *filename;
 	int fd;
 };
 
-#endif	/* USE_BERKELEY_DB */
+/* dbcreate.c */
+struct namedb *namedb_new(const char *filename);
+int namedb_put(struct namedb *db, const uint8_t *dname, struct domain *d);
+int namedb_save(struct namedb *db);
+void namedb_discard(struct namedb *db);
 
-/* Routines for creating the database, dbcreate.c */
-struct namedb *namedb_new __P((char *));
-int namedb_put __P((struct namedb *, u_char *, struct domain *));
-int namedb_save __P((struct namedb *));
-void namedb_discard __P((struct namedb *));
 
-/* Routines for accessing the database, dbaccess.c */
-struct namedb *namedb_open __P((char *));
-struct domain *namedb_lookup __P((struct namedb *, u_char *key));
-struct answer *namedb_answer __P((struct domain *, u_int16_t type));
-void namedb_close __P((struct namedb *));
-
-/* Routines that the calling program must provide... */
-void *xalloc __P((size_t));
-void *xrealloc __P((void *, size_t));
+/* dbaccess.c */
+int domaincmp(const void *a, const void *b);
+unsigned long domainhash(const uint8_t *dname);
+struct domain *namedb_lookup(struct namedb *db, const uint8_t *dname);
+const struct answer *namedb_answer(const struct domain *d, int type);
+struct namedb *namedb_open(const char *filename);
+void namedb_close(struct namedb *db);
 
 #endif
