@@ -1,11 +1,11 @@
 /*
- * $Id: zonec.c,v 1.63.2.6 2002/08/15 14:41:00 alexis Exp $
+ * $Id: zonec.c,v 1.98.2.6 2003/07/21 13:52:07 erik Exp $
  *
  * zone.c -- reads in a zone file and stores it in memory
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -38,122 +38,92 @@
  *
  */
 
+#include <config.h>
+
+#include <assert.h>
+#include <fcntl.h>
+#include <ctype.h>
+#include <errno.h>
+#include <limits.h>
+#include <stdio.h>
+#include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#include <unistd.h>
+
+#include "dname.h"
+#include "dns.h"
+#include "heap.h"
+#include "namedb.h"
+#include "util.h"
 #include "zonec.h"
+#include "zparser.h"
 
-#include <netinet/in.h>		/* htons, htonl on Linux */
-
-static void zone_addbuf __P((struct message *, const void *, size_t));
-static void zone_addcompr __P((struct message *, u_char *, u_int16_t, u_char));
+static void zone_addbuf (struct message *, const void *, size_t);
+static void zone_addcompr (struct message *msg, uint8_t *dname, int offset, int len);
 
 /* The database file... */
-char *dbfile = DEFAULT_DBFILE;
-
-/* The database masks */
-u_char bitmasks[NAMEDB_BITMASKLEN * 3];
-u_char *authmask = bitmasks;
-u_char *starmask = bitmasks + NAMEDB_BITMASKLEN;
-u_char *datamask = bitmasks + NAMEDB_BITMASKLEN * 2;
+static const char *dbfile = DBFILE;
 
 /* Some global flags... */
-int vflag = 0;
-int pflag = 0;
+static int vflag = 0;
+static int pflag = 0;
 
 /* Total errors counter */
-int totalerrors = 0;
+static int totalerrors = 0;
 
-#ifdef	USE_HEAP_HASH
-
-unsigned long
-dnamehash(dname)
-	register u_char *dname;
+static void
+zone_initmsg(struct message *m)
 {
-        register unsigned long hash = 0;
-	register u_char *p = dname;
-
-	dname += *dname + 1;
-
-        while (p < dname)
-                hash = hash * 31 + *p++;
-        return hash;
+	m->ancount = m->nscount = m->arcount = m->dnameslen = m->rrsetslen
+		= m->comprlen = m->pointerslen = m->rrsetsoffslen = 0;
+	m->bufptr = m->buf;
 }
 
-#endif
-
-/*
- * Allocates ``size'' bytes of memory, returns the
- * pointer to the allocated memory or NULL and errno
- * set in case of error. Also reports the error via
- * fprintf(stderr, ...);
- *
- */
-void *
-xalloc(size)
-	register size_t	size;
-{
-	register void *p;
-
-	if((p = malloc(size)) == NULL) {
-		fprintf(stderr, "zonec: malloc failed: %m\n");
-		exit(1);
-	}
-	return p;
-}
-
-void *
-xrealloc(p, size)
-	register void *p;
-	register size_t	size;
-{
-
-	if((p = realloc(p, size)) == NULL) {
-		fprintf(stderr, "zonec: realloc failed: %m\n");
-		exit(1);
-	}
-	return p;
-}
-
-void
-zone_print(z)
-	struct zone *z;
+static void 
+zone_print (struct zone *z)
 {
 	struct rrset *rrset;
-	u_char *dname;
+	struct RR rr;
+	uint8_t *dname;
 	int i;
 
 	printf("; zone %s\n", dnamestr(z->dname));
 	printf("; zone data\n");
 
-	HEAP_WALK(z->data, (char *)dname, rrset) {
+	HEAP_WALK(z->data, dname, rrset) {
 		while(rrset) {
+			rr.dname = dname;
+			rr.ttl = rrset->ttl;
+			rr.class = rrset->class;
+			rr.type = rrset->type;
 			for(i = 0; i < rrset->rrslen; i++) {
-				printf("%s\t%d\t%s\t%s\t", dnamestr(dname), rrset->ttl, \
-					 classtoa(rrset->class), typetoa(rrset->type));
-				zf_print_rdata(rrset->rrs[i], rrset->fmt);
-				printf("\n");
+				rr.rdata = rrset->rrs[i];
+				zprintrr(stdout, &rr);
 			}
 			rrset = rrset->next;
 		}
 	}
 
-	printf("; zone cuts\n");
-	HEAP_WALK(z->cuts, (char *)dname, rrset) {
+	printf("; referrals\n");
+	HEAP_WALK(z->cuts, dname, rrset) {
 		while(rrset) {
+			rr.dname = dname;
+			rr.ttl = rrset->ttl;
+			rr.class = rrset->class;
+			rr.type = rrset->type;
 			for(i = 0; i < rrset->rrslen; i++) {
-				printf("%s\t%d\t%s\t%s\t", dnamestr(dname), rrset->ttl, \
-					 classtoa(rrset->class), typetoa(rrset->type));
-				zf_print_rdata(rrset->rrs[i], rrset->fmt);
-				printf("\n");
+				rr.rdata = rrset->rrs[i];
+				zprintrr(stdout, &rr);
 			}
 			rrset = rrset->next;
 		}
 	}
 }
 
-static void
-zone_addbuf(msg, data, size)
-	struct message *msg;
-	const void *data;
-	size_t size;
+static void 
+zone_addbuf (struct message *msg, const void *data, size_t size)
 {
 	if(msg->bufptr - msg->buf + size > IOBUFSZ) {
 		fflush(stdout);
@@ -161,16 +131,12 @@ zone_addbuf(msg, data, size)
 		exit(1);	/* XXX: do something smart */
 	}
 
-	bcopy(data, msg->bufptr, size);
+	memcpy(msg->bufptr, data, size);
 	msg->bufptr += size;
 }
 
-static void
-zone_addcompr(msg, dname, offset, len)
-	struct message *msg;
-	u_char *dname;
-	u_int16_t offset;
-	u_char len;
+static void 
+zone_addcompr (struct message *msg, uint8_t *dname, int offset, int len)
 {
 	if (msg->comprlen >= MAXRRSPP) {
 		fflush(stdout);
@@ -184,15 +150,13 @@ zone_addcompr(msg, dname, offset, len)
 	msg->comprlen++;
 }
 
-u_int16_t
-zone_addname(msg, dname)
-	struct message *msg;
-	u_char *dname;
+static uint16_t 
+zone_addname (struct message *msg, uint8_t *dname)
 {
 	/* Lets try rdata dname compression */
 	int rdlength = 0;
 	int j;
-	register u_char *t;
+	register uint8_t *t;
 
 	/* Walk through the labels in the dname to be compressed */
 	if(*dname > 1) {
@@ -200,7 +164,8 @@ zone_addname(msg, dname)
 			/* Walk through the dnames that we have already in the packet */
 			for(j = 0; j < msg->comprlen; j++) {
 				if((msg->compr[j].dnamelen == (dname + 1 + *dname - t)) &&
-					(strncasecmp(t, msg->compr[j].dname, msg->compr[j].dnamelen) == 0)) {
+					(strncasecmp((char *)t, (char *)msg->compr[j].dname,
+						msg->compr[j].dnamelen) == 0)) {
 					/* Match, first write down unmatched part */
 					zone_addbuf(msg, dname + 1, t - (dname + 1));
 					rdlength += (t - (dname + 1));
@@ -231,25 +196,19 @@ zone_addname(msg, dname)
 
 
 
-u_int16_t
-zone_addrrset(msg, dname, rrset)
-	struct message *msg;
-	u_char *dname;
-	struct rrset *rrset;
+static uint16_t 
+zone_addrrset (struct message *msg, uint8_t *dname, struct rrset *rrset)
 {
-	u_int16_t class = htons(CLASS_IN);
+	uint16_t class = htons(CLASS_IN);
 	int32_t ttl;
-	union zf_rdatom *rdata;
-	char *rdlengthptr;
-	char *f;
-	size_t size;
-	u_int16_t rdlength;
-	u_int16_t type;
+	uint16_t **rdata;
+	uint8_t *rdlengthptr;
+	uint16_t rdlength;
+	uint16_t type;
 	int rrcount;
 	int i, j;
 
-	u_int32_t l;
-	u_int16_t s;
+	uint16_t s;
 
 	if(rrset == NULL) return 0;
 
@@ -287,7 +246,6 @@ zone_addrrset(msg, dname, rrset)
 		
 		msg->rrsetsoffs[msg->rrsetsoffslen++] = (msg->bufptr - msg->buf) | (rrset->color ? NAMEDB_RRSET_WHITE : 0);
 
-		rdata = rrset->rrs[j];
 
 		/* dname */
 		if(*(dname + 1) == 1 && *(dname + 2) == '*') {
@@ -307,10 +265,10 @@ zone_addrrset(msg, dname, rrset)
 
 		/* type */
 		type = htons(rrset->type);
-		zone_addbuf(msg, &type, sizeof(u_int16_t));
+		zone_addbuf(msg, &type, sizeof(uint16_t));
 
 		/* class */
-		zone_addbuf(msg, &class, sizeof(u_int16_t));
+		zone_addbuf(msg, &class, sizeof(uint16_t));
 
 		/* ttl */
 		ttl = htonl(rrset->ttl);
@@ -319,60 +277,32 @@ zone_addrrset(msg, dname, rrset)
 		/* rdlength */
 		rdlengthptr = msg->bufptr;
 		rdlength = 0;
+
 		/*
 		 * Reserver space for rdata length.  The actual length
 		 * is filled in below.
 		 */
-		zone_addbuf(msg, &rdlength, sizeof(u_int16_t));
+		zone_addbuf(msg, &rdlength, sizeof(uint16_t));
 
 		/* Pack the rdata */
-		for(size = 0, i = 0, f = rrset->fmt; *f; f++, i++, size = 0) {
-			switch(*f) {
-			case '4':
-				size = sizeof(u_int32_t);
-				zone_addbuf(msg, &rdata[i].l, size);
-				break;
-			case 'l':
-				size = sizeof(int32_t);
-				l = htonl(rdata[i].l);
-				zone_addbuf(msg, &l, size);
-				break;
-			case '6':
-				size = IP6ADDRLEN;
-				zone_addbuf(msg, rdata[i].p, size);
-				break;
-			case 'L':
-				size = LOCRDLEN;
-				zone_addbuf(msg, rdata[i].p, size);
-				break;
-			case 'n':
+		
+		for(rdata = rrset->rrs[j]; *rdata; rdata++) {
+			/* Is it a domain name? */
+			if(**rdata == 0xffff) {
 				if (msg->dnameslen >= MAXRRSPP) {
 					fflush(stdout);
 					fprintf(stderr, "zonec: too many domain names\n");
 					exit(1);
 				}
-				
-				size = 0;
-				rdlength += zone_addname(msg, rdata[i].p);
-				msg->dnames[msg->dnameslen++] = rdata[i].p;
-				break;
-			case 't':
-				size = *((u_char *)rdata[i].p) + 1;
-				zone_addbuf(msg, rdata[i].p, size);
-				break;
-			case 's':
-				size = sizeof(u_int16_t);
-				s = htons(rdata[i].s);
-				zone_addbuf(msg, &s, size);
-				break;
-			default:
-				fprintf(stderr, "zonec: panic! uknown atom in format %c\n", *f);
-				return rrcount;
+				rdlength += zone_addname(msg, (uint8_t *)(*rdata + 1));
+				msg->dnames[msg->dnameslen++] = (uint8_t *)(*rdata + 1);
+			} else {
+				zone_addbuf(msg, *rdata + 1, **rdata);
+				rdlength += **rdata;
 			}
-			rdlength += size;
 		}
 		rdlength = htons(rdlength);
-		bcopy(&rdlength, rdlengthptr, sizeof(u_int16_t));
+		memcpy(rdlengthptr, &rdlength, sizeof(uint16_t));
 	}
 	return rrcount;
 }
@@ -381,11 +311,8 @@ zone_addrrset(msg, dname, rrset)
  * Adds an answer to a domain
  *
  */
-struct domain *
-zone_addanswer(d, msg, type)
-	struct domain *d;
-	struct message *msg;
-	u_int16_t type;
+static struct domain *
+zone_addanswer (struct domain *d, struct message *msg, int type)
 {
 	struct answer *a;
 	size_t size, datasize;
@@ -394,12 +321,12 @@ zone_addanswer(d, msg, type)
 	msg->rrsetsoffs[msg->rrsetsoffslen++] = (msg->bufptr - msg->buf);
 
 	datasize = msg->bufptr - msg->buf;
-	size = sizeof(struct answer) + msg->pointerslen * sizeof(u_int16_t) /* ptrs */
-		+ (msg->rrsetsoffslen) * sizeof(u_int16_t)	/* rrs */
+	size = sizeof(struct answer) + msg->pointerslen * sizeof(uint16_t) /* ptrs */
+		+ (msg->rrsetsoffslen) * sizeof(uint16_t)	/* rrs */
 		+ datasize;					/* data */
 
 	/* Assure the alignment for the next answer... */
-	size = ((size + 3) & 0xfffffffc);
+	size = ALIGN_UP(size);
 
 	d = xrealloc(d, d->size + size);
 
@@ -414,9 +341,9 @@ zone_addanswer(d, msg, type)
 	ANSWER_RRSLEN(a) = msg->rrsetsoffslen;
 	ANSWER_DATALEN(a) = datasize;
 
-	bcopy(msg->pointers, ANSWER_PTRS_PTR(a), sizeof(u_int16_t) * msg->pointerslen);
-	bcopy(msg->rrsetsoffs, ANSWER_RRS_PTR(a), sizeof(u_int16_t) * msg->rrsetsoffslen);
-	bcopy(msg->buf, ANSWER_DATA_PTR(a), datasize);
+	memcpy(ANSWER_PTRS_PTR(a), msg->pointers, sizeof(uint16_t) * msg->pointerslen);
+	memcpy(ANSWER_RRS_PTR(a), msg->rrsetsoffs, sizeof(uint16_t) * msg->rrsetsoffslen);
+	memcpy(ANSWER_DATA_PTR(a), msg->buf, datasize);
 
 	d->size += size;
 
@@ -427,9 +354,8 @@ zone_addanswer(d, msg, type)
  * Frees all the data structures associated with the zone
  *
  */
-void
-zone_free(z)
-	struct zone *z;
+static void 
+zone_free (struct zone *z)
 {
 	if(z) {
 		if(z->dname) free(z->dname);
@@ -443,68 +369,70 @@ zone_free(z)
  * Reads the specified zone into the memory
  *
  */
-struct zone *
-zone_read(name, zonefile)
-	char *name;
-	char *zonefile;
+static struct zone *
+zone_read (char *name, char *zonefile)
 {
 	heap_t *h;
 	int i;
+	uint8_t *t, *dname;
 
 	struct zone *z;
-	struct zf *zf;
-	struct zf_entry *rr;
+	struct zparser *parser;
+	struct RR *rr;
 	struct rrset *rrset, *r;
 
 	/* Allocate new zone structure */
 	z = xalloc(sizeof(struct zone));
-	bzero(z, sizeof(struct zone));
+	memset(z, 0, sizeof(struct zone));
 
 	/* Get the zone name */
-	if((z->dname = strdname(name, ROOT_ORIGIN)) == NULL) {
+	if((z->dname = dnamedup(strdname(name, ROOT))) == NULL) {
 		zone_free(z);
 		return NULL;
 	}
 
+#ifndef ROOT_SERVER
+	/* Is it a root zone? Are we a root server then? Idiot proof. */
+	if(dnamecmp(z->dname, ROOT) == 0) {
+		fprintf(stderr, "zonec: Not configured as a root server. See the documentation.\n");
+		zone_free(z);
+		return NULL;
+	}
+#endif
+
 	/* Open the zone file */
-	if((zf = zf_open(zonefile, name)) == NULL) {
+	if((parser = nsd_zopen(zonefile, 3600, CLASS_IN, name)) == NULL) {
+		fprintf(stderr, "zonec: unable to open %s: %s\n", zonefile, strerror(errno));
 		zone_free(z);
 		return NULL;
 	}
 
 	/* Two heaps: zone cuts and other data */
-#ifdef USE_HEAP_RBTREE
 	z->cuts = heap_create(xalloc, dnamecmp);
 	z->data = heap_create(xalloc, dnamecmp);
-#else
-# ifdef USE_HEAP_HASH
-	z->cuts = heap_create(xalloc, dnamecmp, dnamehash, NAMEDB_HASH_SIZE);
-	z->data = heap_create(xalloc, dnamecmp, dnamehash, NAMEDB_HASH_SIZE);
-# endif
-#endif
 	z->soa = z->ns = NULL;
 
 	/* Read the file */
-	while((rr = zf_read(zf)) != NULL) {
+	while((rr = zread(parser)) != NULL) {
 
 		/* Report progress... */
-		if(vflag) {
-			if((zf->lines % 100000) == 0) {
-				printf("zonec: reading zone \"%s\": %d\r", dnamestr(z->dname), zf->lines);
+		if(vflag > 1) {
+			if((parser->lines % 100000) == 0) {
+				printf("zonec: reading zone \"%s\": %lu\r", dnamestr(z->dname), (unsigned long) parser->lines);
 				fflush(stdout);
 			}
 		}
 
 		/* We only support IN class */
 		if(rr->class != CLASS_IN) {
-			zf_error(zf, "wrong class");
+			zerror(parser, "wrong class");
 			continue;
 		}
 
 		/* Is this in-zone data? */
 		if((*z->dname > *rr->dname) ||
-			(bcmp(z->dname + 1, rr->dname + (*rr->dname - *z->dname) + 1, *z->dname) != 0)) {
-			zf_error(zf, "out of zone data");
+			(memcmp(z->dname + 1, rr->dname + (*rr->dname - *z->dname) + 1, *z->dname) != 0)) {
+			zerror(parser, "out of zone data");
 			continue;
 		}
 
@@ -528,61 +456,80 @@ zone_read(name, zonefile)
 		/* Do we have this particular rrset? */
 		if(r == NULL) {
 			r = xalloc(sizeof(struct rrset));
-			bzero(r, sizeof(struct rrset));
+			r->type = 0;
+		}
+		if(r->type == 0) {
+			r->next = NULL;
 			r->type = rr->type;
 			r->class = rr->class;
 			r->ttl = rr->ttl;
-			r->fmt = rr->rdatafmt;
 			r->rrslen = 1;
-			r->rrs = xalloc(sizeof(union zf_rdatom *));
+			r->rrs = xalloc(sizeof(uint16_t *) * 2);
+			r->glue = r->color = 0;
 			r->rrs[0] = rr->rdata;
 
 			/* Add it */
 			if(rrset == NULL) {
 				/* XXX We can use this more smart... */
-				heap_insert(h, strdup(rr->dname), r, 1);
+				heap_insert(h, dnamedup(rr->dname), r, 1);
 			} else {
 				r->next = rrset->next;
 				rrset->next = r;
 			}
 		} else {
 			if(r->ttl != rr->ttl) {
-				zf_error(zf, "rr ttl doesnt match the ttl of the rdataset");
+				zerror(parser, "ttl doesn't match the ttl of the rrset");
 				continue;
 			}
+
 			/* Search for possible duplicates... */
 			for(i = 0; i < r->rrslen; i++) {
-				if(!zf_cmp_rdata(r->rrs[i], rr->rdata, rr->rdatafmt))
+				if(!zrdatacmp(r->rrs[i], rr->rdata))
 					break;
 			}
 
 			/* Discard the duplicates... */
 			if(i < r->rrslen) {
-				/* zf_error(zf, "duplicate record"); */
-				zf_free_rdata(rr->rdata, rr->rdatafmt);
+				/* zerror(parser, "duplicate record"); */
+				zrdatafree(rr->rdata);
 				continue;
 			}
 
 			/* Add it... */
-			r->rrs = xrealloc(r->rrs, ((r->rrslen + 1) * sizeof(union zf_rdatom *)));
+			r->rrs = xrealloc(r->rrs, ((r->rrslen + 2) * sizeof(uint16_t *)));
 			r->rrs[r->rrslen++] = rr->rdata;
+			r->rrs[r->rrslen] = NULL;
 		}
 
+		/* Now create necessary empty nodes... */
+		dname = dnamedup(rr->dname);
+		for(t = dname + 2 + *(dname + 1); (t < (dname + 1 + *dname - *z->dname)); t += *t + 1) {
+			*(t - 1) = dname + *dname - t + 1;
+			if((rrset = heap_search(z->data, t - 1)) == NULL) {
+				r = xalloc(sizeof(struct rrset));
+				memset(r, 0, sizeof(struct rrset));
+
+				/* Add it */
+				heap_insert(z->data, dnamedup(t - 1), r, 1);
+			}
+		}
+		free(dname);
+
 		/* Check we have SOA */
 		if(z->soa == NULL) {
 			if(rr->type != TYPE_SOA) {
-				zf_error(zf, "missing SOA record on top of the zone");
+				zerror(parser, "missing SOA record on top of the zone");
 			} else {
 				if(dnamecmp(rr->dname, z->dname) != 0) {
-					zf_error(zf, "SOA record with invalid domain name");
+					zerror(parser, "SOA record with invalid domain name");
 				} else {
 					z->soa = r;
 				}
 			}
 		} else {
 			if(rr->type == TYPE_SOA) {
-				zf_error(zf, "duplicate SOA record discarded");
-				zf_free_rdata(r->rrs[--r->rrslen], r->fmt);
+				zerror(parser, "duplicate SOA record discarded");
+				zrdatafree(r->rrs[--r->rrslen]);
 			}
 		}
 
@@ -594,18 +541,24 @@ zone_read(name, zonefile)
 	}
 
 	fflush(stdout);
-	fprintf(stderr, "zonec: reading zone \"%s\": %d errors\n", dnamestr(z->dname), zf->errors);
-	totalerrors += zf->errors;
+	if (vflag > 0) {
+		fprintf(stderr, "zonec: reading zone \"%s\": %d errors\n",
+			dnamestr(z->dname), parser->errors);
+	}
+	totalerrors += parser->errors;
+
+	zclose(parser);
+	
 	return z;
 }
 
 static void
-zone_addzonecut(u_char *dkey, u_char *dname, struct rrset *rrset, struct zone *z, struct namedb *db)
+zone_addzonecut(uint8_t *dkey, uint8_t *dname, struct rrset *rrset, struct zone *z, struct namedb *db)
 {
 	struct domain *d;
 	struct message msg;
 	struct rrset *additional;
-	u_char *nameptr;
+	uint8_t *nameptr;
 	int i, namedepth;
 
 	/* Make sure it is not a wildcard */
@@ -615,14 +568,14 @@ zone_addzonecut(u_char *dkey, u_char *dname, struct rrset *rrset, struct zone *z
 	}
 
 	/* Initialize message */
-	bzero(&msg, sizeof(struct message));
-	msg.bufptr = msg.buf;
+	zone_initmsg(&msg);
 
 	/* Create a new domain */
 	d = xalloc(sizeof(struct domain));
 	d->size = sizeof(struct domain);
 	d->flags = NAMEDB_DELEGATION;
-
+	d->runtime_data = NULL;
+	
 	/* Is this a real record? */
 	if(dkey != dname)
 		d->flags |= NAMEDB_STEALTH;
@@ -646,7 +599,7 @@ zone_addzonecut(u_char *dkey, u_char *dname, struct rrset *rrset, struct zone *z
 
 		/* This is a glue record */
 		if((*dkey < *msg.dnames[i]) &&
-		    (bcmp(dkey + 1, msg.dnames[i] + (*msg.dnames[i] - *dkey) + 1, *dkey) == 0)) {
+		    (memcmp(dkey + 1, msg.dnames[i] + (*msg.dnames[i] - *dkey) + 1, *dkey) == 0)) {
 			if(additional == NULL) {
 				fprintf(stderr, "zonec: missing glue record for %s\n", dnamestr(msg.dnames[i]));
 			} else {
@@ -674,9 +627,9 @@ zone_addzonecut(u_char *dkey, u_char *dname, struct rrset *rrset, struct zone *z
 	NAMEDB_SETBITMASK(db, NAMEDB_AUTHMASK, namedepth);
 
 	/* Add a terminator... */
-	d = xrealloc(d, d->size + sizeof(u_int32_t));
-	bzero((char *)d + d->size, sizeof(u_int32_t));
-	d->size += sizeof(u_int32_t);
+	d = xrealloc(d, d->size + sizeof(uint32_t));
+	memset((char *)d + d->size, 0, sizeof(uint32_t));
+	d->size += sizeof(uint32_t);
 
 	/* Store it */
 	if(namedb_put(db, dkey, d) != 0) {
@@ -687,11 +640,11 @@ zone_addzonecut(u_char *dkey, u_char *dname, struct rrset *rrset, struct zone *z
 }
 
 static void
-zone_adddata(u_char *dname, struct rrset *rrset, struct zone *z, struct namedb *db) {
+zone_adddata(uint8_t *dname, struct rrset *rrset, struct zone *z, struct namedb *db) {
 	struct domain *d;
 	struct message msg, msgany;
 	struct rrset *cnamerrset, *additional;
-	u_char *cname, *nameptr;
+	uint8_t *cname, *nameptr;
 	int i, star;
 
 	int namedepth = 0;
@@ -700,116 +653,134 @@ zone_adddata(u_char *dname, struct rrset *rrset, struct zone *z, struct namedb *
 	d = xalloc(sizeof(struct domain));
 	d->size = sizeof(struct domain);
 	d->flags = 0;
+	d->runtime_data = NULL;
 
 	/* This is not a wildcard */
 	star = 0;
 
-	/* Is this a CNAME */
-	if(rrset->type == TYPE_CNAME) {
-		assert(rrset->next == NULL);
-		cnamerrset = rrset;
-		cname = (*cnamerrset->rrs)[0].p;	/* The name of the target set */
-		rrset = heap_search(z->data, cname);
-	} else {
-		cnamerrset = NULL;
-		cname = NULL;
-	}
-
-	/* Initialize message for TYPE_ANY */
-	bzero(&msgany, sizeof(struct message));
-	msgany.bufptr = msgany.buf;
-
-	/* XXX This is a bit confusing, needs renaming:
-	 *
-	 * cname - name of the target set
-	 * rrset - target rr set
-	 * cnamerrset - cname own rrset 
-	 * dname - cname's rrset owner name
-	 */
-	while(rrset || cnamerrset) {
-		/* Initialize message */
-		bzero(&msg, sizeof(struct message));
-		msg.bufptr = msg.buf;
-
-		/* If we're done with the target sets, add CNAME itself */
-		if(rrset == NULL) {
-			rrset = cnamerrset;
+	/* Node with data? */
+	if(rrset->type != 0) {
+		/* Is this a CNAME */
+		if(rrset->type == TYPE_CNAME) {
+			/* XXX Not necessarily with NXT, BUT OH OH * assert(rrset->next == NULL); */
+			cnamerrset = rrset;
+			cname = (uint8_t *)(*cnamerrset->rrs[0]+1);
+			rrset = heap_search(z->data, cname);
+		} else {
 			cnamerrset = NULL;
+			cname = NULL;
 		}
 
-		/* Put the dname into compression array */
-		for(namedepth = 0, nameptr = dname + 1; *nameptr; nameptr += *nameptr + 1, namedepth++) {
-			/* Do we have a wildcard? */
-			if((namedepth == 0) && (*(nameptr+1) == '*')) {
-				star = 1;
-			} else {
-				if((dname + *dname + 1 - nameptr) > 1) {
-					zone_addcompr(&msg, nameptr,
-						      (nameptr - (dname + 1)) | 0xc000,
-						      dname + *dname + 1 - nameptr);
-					zone_addcompr(&msgany, nameptr,
-						      (nameptr - (dname + 1)) | 0xc000,
-						      dname + *dname + 1 - nameptr);
+		/* Initialize message for TYPE_ANY */
+		zone_initmsg(&msgany);
+
+		/* XXX This is a bit confusing, needs renaming:
+		 *
+		 * cname - name of the target set
+		 * rrset - target rr set
+		 * cnamerrset - cname own rrset 
+		 * dname - cname's rrset owner name
+		 */
+		while(rrset || cnamerrset) {
+			/*
+			 * When a CNAME points to a SOA record, don't
+			 * add it to the answer.  Otherwise bug #56
+			 * gets triggered because AXFR terminates
+			 * early (because it thinks the final SOA
+			 * record was encountered).
+			 */
+			if (cnamerrset && rrset && rrset->type == TYPE_SOA) {
+				rrset = rrset->next;
+				continue;
+			}
+			
+			/* Initialize message */
+			zone_initmsg(&msg);
+
+			/* If we're done with the target sets, add CNAME itself */
+			if(rrset == NULL) {
+				rrset = cnamerrset;
+				cnamerrset = NULL;
+			}
+
+			/* Put the dname into compression array */
+			for(namedepth = 0, nameptr = dname + 1; *nameptr; nameptr += *nameptr + 1, namedepth++) {
+				/* Do we have a wildcard? */
+				if((namedepth == 0) && (*(nameptr+1) == '*')) {
+					star = 1;
+				} else {
+					if((dname + *dname + 1 - nameptr) > 1) {
+						zone_addcompr(&msg, nameptr,
+							      (nameptr - (dname + 1)) | 0xc000,
+							      dname + *dname + 1 - nameptr);
+						zone_addcompr(&msgany, nameptr,
+							      (nameptr - (dname + 1)) | 0xc000,
+							      dname + *dname + 1 - nameptr);
+					}
 				}
 			}
+
+			/* Are we doing CNAME? */
+			if(cnamerrset) {
+				/* Add CNAME itself */
+				msg.ancount += zone_addrrset(&msg, dname, cnamerrset);
+
+				/* Add answer */
+				msg.ancount += zone_addrrset(&msg, cname, rrset);
+			} else {
+				/* Answer section */
+				msg.ancount += zone_addrrset(&msg, dname, rrset);
+
+				/* Answer section of message any */
+				msgany.ancount += zone_addrrset(&msgany, dname, rrset);
+			}
+
+			/* Authority section */
+			msg.nscount = zone_addrrset(&msg, z->dname, z->ns);
+
+			/* Additional section */
+			for(i = 0; i < msg.dnameslen; i++) {
+				additional = heap_search(z->data, msg.dnames[i]);
+				while(additional) {
+					if(additional->type == TYPE_A || additional->type == TYPE_AAAA) {
+						msg.arcount += zone_addrrset(&msg, msg.dnames[i], additional);
+					}
+					additional = additional->next;
+				}
+			}
+
+			/* Add this answer */
+			d = zone_addanswer(d, &msg, rrset->type);
+
+			/* Set the masks */
+			if(rrset->type == TYPE_SOA)
+				NAMEDB_SETBITMASK(db, NAMEDB_AUTHMASK, namedepth);
+
+			rrset = rrset->next;
 		}
 
-		/* Are we doing CNAME? */
-		if(cnamerrset) {
-			/* Add CNAME itself */
-			msg.ancount += zone_addrrset(&msg, dname, cnamerrset);
+		/* Authority section for TYPE_ANY */
+		msgany.nscount = zone_addrrset(&msgany, z->dname, z->ns);
 
-			/* Add answer */
-			msg.ancount += zone_addrrset(&msg, cname, rrset);
-		} else {
-			/* Answer section */
-			msg.ancount += zone_addrrset(&msg, dname, rrset);
-
-			/* Answer section of message any */
-			msgany.ancount += zone_addrrset(&msgany, dname, rrset);
-		}
-
-		/* Authority section */
-		msg.nscount = zone_addrrset(&msg, z->dname, z->ns);
-
-		/* Additional section */
-		for(i = 0; i < msg.dnameslen; i++) {
-			additional = heap_search(z->data, msg.dnames[i]);
+		/* Additional section for TYPE_ANY */
+		for(i = 0; i < msgany.dnameslen; i++) {
+			additional = heap_search(z->data, msgany.dnames[i]);
 			while(additional) {
 				if(additional->type == TYPE_A || additional->type == TYPE_AAAA) {
-					msg.arcount += zone_addrrset(&msg, msg.dnames[i], additional);
+					msgany.arcount += zone_addrrset(&msgany, msgany.dnames[i], additional);
 				}
 				additional = additional->next;
 			}
 		}
 
 		/* Add this answer */
-		d = zone_addanswer(d, &msg, rrset->type);
-
-		/* Set the masks */
-		if(rrset->type == TYPE_SOA)
-			NAMEDB_SETBITMASK(db, NAMEDB_AUTHMASK, namedepth);
-
-		rrset = rrset->next;
+		d = zone_addanswer(d, &msgany, TYPE_ANY);
+	} else {
+		/* This is an empty node...*/
+		d->flags |= NAMEDB_STEALTH;
+		for(namedepth = 0, nameptr = dname + 1; *nameptr; nameptr += *nameptr + 1, namedepth++);
 	}
 
-	/* Authority section for TYPE_ANY */
-	msgany.nscount = zone_addrrset(&msgany, z->dname, z->ns);
-
-	/* Additional section for TYPE_ANY */
-	for(i = 0; i < msgany.dnameslen; i++) {
-		additional = heap_search(z->data, msgany.dnames[i]);
-		while(additional) {
-			if(additional->type == TYPE_A || additional->type == TYPE_AAAA) {
-				msgany.arcount += zone_addrrset(&msgany, msgany.dnames[i], additional);
-			}
-			additional = additional->next;
-		}
-	}
-
-	/* Add this answer */
-	d = zone_addanswer(d, &msgany, TYPE_ANY);
-
 	/* Set the data mask */
 	NAMEDB_SETBITMASK(db, NAMEDB_DATAMASK, namedepth);
 	if(star) {
@@ -817,9 +788,9 @@ zone_adddata(u_char *dname, struct rrset *rrset, struct zone *z, struct namedb *
 	}
 
 	/* Add a terminator... */
-	d = xrealloc(d, d->size + sizeof(u_int32_t));
-	bzero((char *)d + d->size, sizeof(u_int32_t));
-	d->size += sizeof(u_int32_t);
+	d = xrealloc(d, d->size + sizeof(uint32_t));
+	memset((char *)d + d->size, 0, sizeof(uint32_t));
+	d->size += sizeof(uint32_t);
 
 	/* Store it */
 	if(namedb_put(db, dname, d) != 0) {
@@ -834,14 +805,12 @@ zone_adddata(u_char *dname, struct rrset *rrset, struct zone *z, struct namedb *
  *
  * Returns zero if success.
  */
-int
-zone_dump(z, db)
-	struct 	zone *z;
-	struct namedb *db;
+static int 
+zone_dump (struct zone *z, struct namedb *db)
 {
-	u_char dnamebuf[MAXDOMAINLEN+1];
+	uint8_t dnamebuf[MAXDOMAINLEN+1];
 	struct rrset *rrset;
-	u_char *dname, *nameptr;
+	uint8_t *dname, *nameptr;
 	
 	/* Progress reporting... */
 	unsigned long progress = 0;
@@ -849,7 +818,7 @@ zone_dump(z, db)
 	int percentage = 0;
 
 	/* Set up the counter... */
-	if(vflag) {
+	if(vflag > 1) {
 		fraction = (z->cuts->count + z->data->count) / 20;	/* Report every 5% */
 		if(fraction == 0)
 			fraction = ULONG_MAX;
@@ -865,9 +834,9 @@ zone_dump(z, db)
 	}
 
 	/* AUTHORITY CUTS */
-	HEAP_WALK(z->cuts, (char *)dname, rrset) {
+	HEAP_WALK(z->cuts, dname, rrset) {
 		/* Report progress... */
-		if(vflag) {
+		if(vflag > 1) {
 			if((++progress % fraction) == 0) {
 				printf("zonec: writing zone \"%s\": %d%%\r", dnamestr(z->dname), percentage);
 				percentage += 5;
@@ -886,9 +855,9 @@ zone_dump(z, db)
 	}
 
 	/* OTHER DATA */
-	HEAP_WALK(z->data, (char *)dname, rrset) {
+	HEAP_WALK(z->data, dname, rrset) {
 		/* Report progress... */
-		if(vflag) {
+		if(vflag > 1) {
 			if((++progress % fraction) == 0) {
 				printf("zonec: writing zone \"%s\": %d%%\r", dnamestr(z->dname), percentage);
 				percentage += 5;
@@ -905,7 +874,7 @@ zone_dump(z, db)
 			continue;
 
 		/* This is an ugly slow way to find out of zone data... */
-		bcopy(dname, dnamebuf, *dname + 1);
+		memcpy(dnamebuf, dname, *dname + 1);
 		for(nameptr = dnamebuf + 1; *(nameptr - 1) > *z->dname; 
 				nameptr += *nameptr + 1, *(nameptr - 1) = dnamebuf + *dnamebuf - nameptr + 1) {
 
@@ -922,9 +891,13 @@ zone_dump(z, db)
 		/* CNAME & other data */
 		if((rrset->type == TYPE_CNAME && rrset->next != NULL) ||
 			(rrset->next != NULL && rrset->next->type == TYPE_CNAME)) {
-			fprintf(stderr, "CNAME and other data for %s\n", dnamestr(z->dname));
-			totalerrors++;
-			continue;
+			if(rrset->type != TYPE_NXT && rrset->next->type != TYPE_NXT) {
+				if(rrset->type != TYPE_SIG && rrset->next->type != TYPE_SIG) {
+					fprintf(stderr, "CNAME and other data for %s\n", dnamestr(z->dname));
+					totalerrors++;
+					continue;
+				}
+			}
 		}
 
 		/* Add it to the database */
@@ -932,13 +905,16 @@ zone_dump(z, db)
 	}
 
 	fflush(stdout);
-	fprintf(stderr, "zonec: writing zone \"%s\": done.\n", dnamestr(z->dname));
+	if (vflag > 0) {
+		fprintf(stderr, "zonec: writing zone \"%s\": done.\n",
+			dnamestr(z->dname));
+	}
 
 	return 0;
 }
 
-int
-usage()
+static void 
+usage (void)
 {
 	fprintf(stderr, "usage: zonec [-f database] [-d directory] zone-list-file\n");
 	exit(1);
@@ -947,21 +923,21 @@ usage()
 extern char *optarg;
 extern int optind;
 
-int
-main(argc, argv)
-	int argc;
-	char **argv;
+int 
+main (int argc, char **argv)
 {
 	char *zonename, *zonefile, *s;
 	char buf[LINEBUFSZ];
         struct namedb *db;
-	char *sep = " \t\n";
+	const char *sep = " \t\n";
 	int c;
 	int line = 0;
 	FILE *f;
 
 	struct zone *z = NULL;
 
+	log_init("zonec");
+	
 	totalerrors = 0;
 
 	/* Parse the command line... */
@@ -971,7 +947,7 @@ main(argc, argv)
 			pflag = 1;
 			break;
 		case 'v':
-			vflag = 1;
+			++vflag;
 			break;
 		case 'f':
 			dbfile = optarg;
