@@ -1,11 +1,11 @@
 /*
- * $Id: zonec.h,v 1.16 2002/05/23 13:33:03 alexis Exp $
+ * $Id: zonec.h,v 1.25.2.2 2003/07/21 13:52:09 erik Exp $
  *
  * zone.h -- internal zone representation
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -38,44 +38,22 @@
  *
  */
 
-#include "config.h"
-
-#include <sys/types.h>
-#include <sys/param.h>
-
-#include <assert.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <limits.h>
-#include <stddef.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <syslog.h>
-#include <unistd.h>
-
-#include "heap.h"
-#include "dns.h"
-#include "zf.h"
-#include "namedb.h"
-
-#define	DEFAULT_DBFILE	"nsd.db"
+#ifndef _ZONEC_H_
+#define _ZONEC_H_
 
 struct rrset {
 	struct rrset *next;
-	u_int16_t type;
-	u_int16_t class;
+	uint16_t type;
+	uint16_t class;
 	int32_t ttl;
-	char *fmt;
-	u_int16_t rrslen;
 	int glue;
 	int color;
-	union zf_rdatom **rrs;
+	uint16_t rrslen;
+	uint16_t ***rrs;
 };
 
 struct zone {
-	u_char *dname;
+	uint8_t *dname;
 	heap_t	*cuts;
 	heap_t	*data;
 	struct rrset *soa;
@@ -84,25 +62,28 @@ struct zone {
 
 #define MAXRRSPP	1024
 #define	IOBUFSZ		MAXRRSPP * 64
+#define	LINEBUFSZ	1024
 
 struct message {
-	u_char *bufptr;
-	u_int16_t ancount;
-	u_int16_t nscount;
-	u_int16_t arcount;
+	uint8_t *bufptr;
+	uint16_t ancount;
+	uint16_t nscount;
+	uint16_t arcount;
 	int dnameslen;
 	int rrsetslen;
 	int comprlen;
-	u_int16_t pointerslen;
-	u_int16_t pointers[MAXRRSPP];
-	u_int16_t rrsetsoffslen;
-	u_int16_t rrsetsoffs[MAXRRSPP];
+	uint16_t pointerslen;
+	uint16_t pointers[MAXRRSPP];
+	uint16_t rrsetsoffslen;
+	uint16_t rrsetsoffs[MAXRRSPP];
 	struct rrset *rrsets[MAXRRSPP];
-	u_char *dnames[MAXRRSPP];
+	uint8_t *dnames[MAXRRSPP];
 	struct {
-		u_char *dname;
-		u_int16_t dnameoff;
-		u_char dnamelen;
+		uint8_t *dname;
+		uint16_t dnameoff;
+		uint8_t dnamelen;
 	} compr[MAXRRSPP];
-	u_char buf[IOBUFSZ];
+	uint8_t buf[IOBUFSZ];
 };
+
+#endif /* _ZONEC_H_ */
